import petl
from petl import rowslice, tohtml
from petl.io import StringSource
from petlx.util import UnsatisfiedDependency


dep_message = """
iPython is required. Instructions for installation can be found 
at http://ipython.org/install.html or try apt-get install 
ipython-notebook.
"""


def display(tbl, *sliceargs):
    """
    Display a table inline within an iPython notebook. E.g.::
    
        In [0]: from petlx.ipython import display
                tbl = [['foo', 'bar'], ['a', 1], ['b', 2]]
                display(tbl)
                
    Alternatively, using the fluent style::
    
        In [0]: from petl.interactive import etl
                import petlx.ipython
                tbl = [['foo', 'bar'], ['a', 1], ['b', 2]]
                etl(tbl).display()
                
    .. versionadded:: 0.5  
    
    .. deprecated:: 0.6
    
    The :mod:`petl.interactive` module supports `_repr_html_` as of 0.13.1 so
    this function is not necessary. E.g., the following should give an HTML 
    rendering of the table inline within an iPython notebook::

        In [0]: from petl.interactive import etl
                tbl = [['foo', 'bar'], ['a', 1], ['b', 2]]
                etl(tbl)
        
    """
    try:
        from IPython.core.display import display_html
    except ImportError as e:
        raise UnsatisfiedDependency(e, dep_message)        
    if not sliceargs:
        sliceargs = (10,)
    tbl = rowslice(tbl, *sliceargs)
    if petl.interactive.InteractiveWrapper.repr_index_header:
        indexed_header = ['%s|%s' % (i, f) for (i, f) in enumerate(petl.util.header(tbl))]
        tbl = petl.transform.setheader(tbl, indexed_header)
    buf = StringSource()
    tohtml(tbl, buf)
    display_html(buf.getvalue(), raw=True)


def displayall(tbl):
    """
    Display *all rows* from a table inline within an iPython notebook. E.g.::
    
        In [0]: from petlx.ipython import displayall
                tbl = [['foo', 'bar'], ['a', 1], ['b', 2]]
                displayall(tbl)
                
    Alternatively, using the fluent style::
    
        In [0]: from petl.interactive import etl
                import petlx.ipython
                tbl = [['foo', 'bar'], ['a', 1], ['b', 2]]
                etl(tbl).displayall()
                
    .. versionadded:: 0.5  

    .. deprecated:: 0.6
    
    The :mod:`petl.interactive` module supports `_repr_html_` as of 0.13.1 so
    this function is not necessary. E.g., the following should give an HTML 
    rendering of the table inline within an iPython notebook::

        In [0]: from petl.interactive import etl
                tbl = [['foo', 'bar'], ['a', 1], ['b', 2]]
                etl(tbl)        
        
    """
    try:
        from IPython.core.display import display_html
    except ImportError as e:
        raise UnsatisfiedDependency(e, dep_message)        
    if petl.interactive.InteractiveWrapper.repr_index_header:
        indexed_header = ['%s|%s' % (i, f) for (i, f) in enumerate(petl.util.header(tbl))]
        tbl = petl.transform.setheader(tbl, indexed_header)
    buf = StringSource()
    tohtml(tbl, buf)
    display_html(buf.getvalue(), raw=True)


import sys
from .integration import integrate
integrate(sys.modules[__name__])




import petl.fluent as fluent
import petl.interactive as interactive


def integrate(mod):
    # integrate with petl.fluent
    for n, c in mod.__dict__.items():
        if callable(c):
            if n.startswith('from'): # avoids having to import anything other than "etl"
                setattr(fluent.FluentWrapper, n, staticmethod(fluent.wrap(c)))
            else:
                setattr(fluent.FluentWrapper, n, fluent.wrap(c)) 
      
    # integrate with petl.interactive
    for n, c in mod.__dict__.items():
        if callable(c):
            if n.startswith('from'): # avoids having to import anything other than "etl"
                setattr(interactive.InteractiveWrapper, n, staticmethod(interactive.wrap(c)))
            else:
                setattr(interactive.InteractiveWrapper, n, interactive.wrap(c)) 
          



from petl.util import RowContainer
from petlx.util import UnsatisfiedDependency


dep_message = """
The package pysam is required. Instructions for installation can be found 
at https://code.google.com/p/pysam/ or try pip install pysam.
"""


def fromtabix(filename, reference=None, start=None, end=None, region=None, header=None):
    """
    Extract rows from a tabix indexed file. E.g.::
    
        >>> from petlx.tabix import fromtabix
        >>> from petl import look
        >>> t = fromtabix('test.bed.gz', region='Pf3D7_02_v3:100000-200000')
        >>> look(t)
        +---------------+----------+----------+-----------------------------+
        | '#chrom'      | 'start'  | 'end'    | 'region'                    |
        +===============+==========+==========+=============================+
        | 'Pf3D7_02_v3' | '23100'  | '105800' | 'SubtelomericHypervariable' |
        +---------------+----------+----------+-----------------------------+
        | 'Pf3D7_02_v3' | '105800' | '447300' | 'Core'                      |
        +---------------+----------+----------+-----------------------------+
    
    .. versionadded:: 0.4
    
    """
    
    return TabixContainer(filename, reference, start, end, region, header)


class TabixContainer(RowContainer):
    
    def __init__(self, filename, reference=None, start=None, end=None, region=None, header=None):
        self.filename = filename
        self.reference = reference
        self.start = start
        self.end = end
        self.region = region
        self.header = header
        
    def __iter__(self):
        try:
            from pysam import Tabixfile, asTuple
        except ImportError as e:
            raise UnsatisfiedDependency(e, dep_message)
        f = Tabixfile(self.filename, mode='r')
        try:
            # header row
            if self.header is not None:
                yield self.header
            else:
                # assume last header line has fields
                h = list(f.header)
                if len(h) > 0:
                    yield tuple(h[-1].split('\t'))
            # data rows
            for row in f.fetch(reference=self.reference, start=self.start, end=self.end, region=self.region, parser=asTuple()):
                yield tuple(row)
        except:
            raise
        finally:
            f.close()


import sys    
from .integration import integrate
integrate(sys.modules[__name__])




"""
Utilities for working with GFF3 files.

"""


from petl.io import fromtsv
from petl.transform import skipcomments, rowlenselect, convert, pushheader
from urllib import unquote_plus
from petl.util import HybridRow, RowContainer
from petlx.interval import facetintervallookup, intervaljoin, intervalleftjoin
import sys


def gff3_parse_attributes(attributes_string):
    """
    Parse a string of GFF3 attributes ('key=value' pairs delimited by ';') 
    and return a dictionary.
  
    .. versionadded:: 0.2
      
    """
    
    attributes = dict()
    fields = attributes_string.split(';')
    for f in fields:
        if '=' in f:
            key, value = f.split('=')
            attributes[unquote_plus(key).strip()] = unquote_plus(value.strip())
        elif len(f) > 0:
            # not strictly kosher
            attributes[unquote_plus(f).strip()] = True            
    return attributes


def fromgff3(filename):
    """
    Extract feature rows from a GFF3 file. 
    
    .. versionadded:: 0.2

    """
    
    # parse file as tab-delimited
    t0 = fromtsv(filename)
    
    # push header
    t1 = pushheader(t0, ('seqid', 'source', 'type', 'start', 'end', 'score', 'strand', 'phase', 'attributes'))

    # skip comments
    t2 = skipcomments(t1, '#')
    
    # ignore any row not 9 values long (e.g., trailing fasta)
    t3 = rowlenselect(t2, 9)
    
    # parse attributes into a dict
    t4 = convert(t3, 'attributes', gff3_parse_attributes)
    
    # parse coordinates
    t5 = convert(t4, ('start', 'end'), int)

    return HybridRowView(t5)


# TODO move this to petl.base?
class HybridRowView(RowContainer):
    
    def __init__(self, wrapped):
        self.wrapped = wrapped
        
    def cachetag(self):
        return self.wrapped.cachetag()
    
    def __iter__(self):
        it = iter(self.wrapped)
        fields = it.next()
        yield fields
        for row in it:
            yield HybridRow(row, fields, missing=None)
            
            
def gff3lookup(features, facet='seqid'):
    """
    Build a GFF3 feature lookup based on interval trees. See also 
    :func:`petlx.interval.facetintervallookup`.
    
    .. versionadded:: 0.2
    
    """
    
    return facetintervallookup(features, key=facet, start='start', stop='end')


def gff3join(table, features, seqid='seqid', start='start', end='end', proximity=1):
    """
    Join with a table of GFF3 features. See also :func:`petlx.interval.intervaljoin`.
    
    .. versionadded:: 0.2
    
    """
    
    return intervaljoin(table, features, lstart=start, lstop=end, lfacet=seqid,
                        rstart='start', rstop='end', rfacet='seqid', 
                        proximity=proximity)

    
def gff3leftjoin(table, features, seqid='seqid', start='start', end='end', proximity=1):
    """
    Left join with a table of GFF3 features. See also :func:`petlx.interval.intervalleftjoin`.
    
    .. versionadded:: 0.2
    
    """
    
    return intervalleftjoin(table, features, lstart=start, lstop=end, lfacet=seqid,
                            rstart='start', rstop='end', rfacet='seqid', 
                            proximity=proximity)


from .integration import integrate
integrate(sys.modules[__name__])
    



from ast import literal_eval
from distutils.core import setup


def get_version(source='src/petlx/__init__.py'):
    with open(source) as f:
        for line in f:
            if line.startswith('VERSION'):
                return literal_eval(line.partition('=')[2].lstrip())
    raise ValueError("VERSION not found")


setup(
    name='petlx',
    version=get_version(),
    author='Alistair Miles',
    author_email='alimanfoo@googlemail.com',
    package_dir={'': 'src'},
    packages=['petlx'],
    url='https://github.com/alimanfoo/petlx',
    license='MIT License',
    description='Optional extensions for the petl package.',
    long_description=open('README.txt').read(),
    classifiers=['Intended Audience :: Developers',
                 'License :: OSI Approved :: MIT License',
                 'Programming Language :: Python',
                 'Topic :: Software Development :: Libraries :: Python Modules'
                 ],
    requires=['petl (>=0.12)']  
)
